==> tdf?file=1&path=frealign_v8.11_130826.tar.gz: frealign_v8.11/src/pinsert.f <==
C**************************************************************************
      SUBROUTINE PINSERT(NSAM,IRAD,RI,RREC,AMAG,SPEC,SPEQ,CTFF,
     +		   CTFS,A3DF,A3DS,S3DF,S3DS,B3DF,B3DS,T3DF,T3DS,
     +		   TTD,PHI,THETA,PSI,SHX,SHY,PRESA,PBC,BOFF,
     +		   ASUM,VSUM,PSUM,C3DF,KSUM,ISIGN,SINCLUT,IRADA,
     +             IPAD,THET,IEWALD,C3DS,NSYM,ISYM,JSYM,SYMOP,
     +             NNSTAT,NKSUM,IMP,NKSUM1,TTD1,NS,NS1)
C**************************************************************************
C  Adds an image transform into the accumulating 3D transform and also 
C  accumulates the CTF**2 for use in later normalisation.
C  Uses PINSERT_S.
C  Used in A3D3.
C**************************************************************************
      IMPLICIT NONE

      INTEGER NSAM,JC,I,J,IRAD,IC,IEWALD,NS1(*),NS(*)
      INTEGER L,ISIGN,NSAMH,ID,IS,IRREC,IRREC2,NSYM,IMP
      INTEGER KSUM(*),IPAD,IRADA,ISYM(*),JSYM(*),NNSTAT
      INTEGER NL,NJ
      INTEGER*8 NKSUM,NKSUM1(*)
      REAL DM(9),SHX,SHY,RREC,TLIM,TM(9),SYMOP(3,3,*)
      REAL CTFV2,CTFV2S2,S3DF(*),CPHI,SPHI,RI,ST
      REAL CTHE,STHE,CPSI,SPSI,PHI,THETA,PSI
      REAL PHASE,PBC,BOFF,WGT,PI,TMP,AMAG
      REAL S3DS(*),RII,PREF,T3DF(*),T3DS(*),PRESA
      REAL VSUM(*),PSUM(*),ASUM(*),SINCLUT(*),THET
      DOUBLEPRECISION TTD,TTD1(*)
      PARAMETER (PI=3.1415926535897)
      PARAMETER (TLIM=2.0)
      COMPLEX SAMP,SPEC(*),SPEQ(*),BSAMP,CTFF(*),CTFS(*)
      COMPLEX A3DF(*),A3DS(*),B3DF(*),B3DS(*)
      COMPLEX C3DF(*),C3DS(*)
C**************************************************************************
      	PREF=PBC/180.0*PI
        JC=NSAM/2+1
        IC=NSAM*NSAM/2
      	NSAMH=NSAM/2
      	RII=2.0*PI*RI/NSAM
      	IRREC=INT(REAL(NSAM)*RREC)
C      	IF (IRREC.GT.NSAMH) IRREC=NSAMH
      	IRREC2=IRREC**2
        CPHI=COS(PHI)
        SPHI=SIN(PHI)
        CTHE=COS(THETA)
        STHE=SIN(THETA)
        CPSI=COS(PSI)
        SPSI=SIN(PSI)
c      PRINT *,PRESA*180.0/PI,PBC,BOFF/PBC,-(PRESA/PREF-BOFF/PBC)
      	TMP=-(PRESA/PREF-BOFF/PBC)
        IF (TMP.GT.TLIM) THEN
      	  TMP=TLIM
C      	  PRINT *,'TLIM IN ACTION!!!!!'
      	ENDIF
      	TMP=2.0*TMP/REAL(NSAMH)**2
C      	WGT=EXP(-PRESA/PREF)

        TM(1)=(CPHI*CTHE*CPSI-SPHI*SPSI)/ABS(AMAG)
        TM(2)=(SPHI*CTHE*CPSI+CPHI*SPSI)/ABS(AMAG)
        TM(3)=-STHE*CPSI/ABS(AMAG)
        TM(4)=(-CPHI*CTHE*SPSI-SPHI*CPSI)/ABS(AMAG)
        TM(5)=(-SPHI*CTHE*SPSI+CPHI*CPSI)/ABS(AMAG)
        TM(6)=STHE*SPSI/ABS(AMAG)
        TM(7)=STHE*CPHI/ABS(AMAG)
        TM(8)=STHE*SPHI/ABS(AMAG)
        TM(9)=CTHE/ABS(AMAG)
C
      	IF (ISIGN.LT.0) THEN
      	  DO 10 L=1,NSAM
      	    NL=NSAM*(L-1)
      	    DO 10 J=1,NSAM
      	      IS=J+NL
      	      NJ=NSAMH*((J-1)+NL)
      	      B3DS(IS)=A3DS(IS)
c      	      T3DS(IS)=S3DS(IS)
      	      DO 10 I=1,NSAMH
      	        ID=I+NJ
      	        B3DF(ID)=A3DF(ID)
c      	        T3DF(ID)=S3DF(ID)
10	  CONTINUE
      	ENDIF
C
        DO 940 I=1,NSYM
          ISYM(I)=1
940     CONTINUE
C
900     CONTINUE
C
        DO 910 I=1,9
          DM(I)=TM(I)
910     CONTINUE
        DO 920 I=1,NSYM
          IF (ISYM(I).NE.JSYM(I)) THEN
            DO 930 J=1,ISYM(I)
              CALL MATMUL_T(DM,SYMOP(1,1,I),DM)
930         CONTINUE
          ENDIF
920     CONTINUE
C
        ST=REAL(2*JC-1)/REAL(IMP)
        DO 800 I=1,IMP
          NKSUM1(I)=0
          TTD1(I)=0.0D0
800     CONTINUE
C NS, NS1 used to count measurements in each resolution shell
C Not needed at the moment and therefore commented out here and
C in PINSERT_S.
C          DO 820 I=1,JC*IMP
C            NS1(I)=0
C820       CONTINUE
!$OMP PARALLEL DO
        DO 20 I=1,IMP
          CALL PINSERT_S(NSAM,IRAD,RII,IRREC2,SPEC,SPEQ,CTFF,
     +             CTFS,B3DF,B3DS,T3DF,T3DS,TTD1,SHX,SHY,
     +             ASUM,VSUM,PSUM,C3DF,KSUM,ISIGN,SINCLUT,IRADA,
     +             IPAD,THET,IEWALD,C3DS,NNSTAT,NKSUM1(I),JC,DM,
     +             TMP,NSAMH,IC,0.0,0.0,ST,I,NS1)
20      CONTINUE
        DO 810 I=1,IMP
          NKSUM=NKSUM+NKSUM1(I)
          TTD=TTD+TTD1(I)
C            DO 810 L=1,JC
C              ID=JC*(I-1)+L
C              NS(L)=NS(L)+NS1(ID)
810     CONTINUE
C
        I=1
990     CONTINUE
        ISYM(I)=ISYM(I)+1
        IF (ISYM(I).GT.JSYM(I)) THEN
          ISYM(I)=1
          I=I+1
          IF (I.LE.NSYM) GOTO 990
        ELSE
          I=1
          GOTO 900
        ENDIF
C
      	RETURN
      	END

==> tdf?file=1&path=frealign_v9.05_131002.tar.gz: frealign_v9.05/src/pinsert.f <==
C*
C* Copyright 2013 Howard Hughes Medical Institute.
C* All rights reserved.
C* Use is subject to Janelia Farm Research Campus Software Copyright 1.1
C* license terms ( http://license.janelia.org/license/jfrc_copyright_1_1.html )
C*
C**************************************************************************
      SUBROUTINE PINSERT(NSAM,IRAD,RI,RREC,AMAG,SPEC,SPEQ,CTFF,
     +		   CTFS,A3DF,A3DS,S3DF,S3DS,B3DF,B3DS,T3DF,T3DS,
     +		   TTD,PHI,THETA,PSI,SHX,SHY,PRESA,PBC,BOFF,
     +		   ASUM,VSUM,PSUM,C3DF,KSUM,ISIGN,SINCLUT,IRADA,
     +             IPAD,THET,IEWALD,C3DS,NSYM,ISYM,
     +             JSYM,SYMOP,NNSTAT,NKSUM,IMP,OCC,NKSUM1,TTD1)
C**************************************************************************
C  Adds an image transform into the accumulating 3D transform and also 
C  accumulates the CTF**2 for use in later normalisation.
C  Uses PINSERT_S.
C  Used in A3D3.
C**************************************************************************
      IMPLICIT NONE

      INTEGER NSAM,JC,I,J,IRAD,IC,IEWALD,NL,NJ
      INTEGER L,ISIGN,NSAMH,ID,IS,IRREC,IRREC2,NSYM,IMP
      INTEGER KSUM(*),IPAD,IRADA,ISYM(*),JSYM(*),NNSTAT
      INTEGER*8 NKSUM,NKSUM1(*)
      REAL DM(9),SHX,SHY,RREC,TLIM,TM(9),SYMOP(3,3,*)
      REAL CTFV2,CTFV2S2,S3DF(*),RI,ST
      REAL PHI,THETA,PSI,OCC
      REAL PHASE,PSHFT,PBC,BOFF,PI,TMP,AMAG
      REAL S3DS(*),RII,PREF,T3DF(*),T3DS(*),PRESA
      REAL VSUM(*),PSUM(*),ASUM(*),SINCLUT(*),THET
      DOUBLEPRECISION TTD,TTD1(*)
      PARAMETER (PI=3.1415926535897)
      PARAMETER (TLIM=2.0)
      COMPLEX SAMP,SPEC(*),SPEQ(*),BSAMP,CTFF(*),CTFS(*)
      COMPLEX A3DF(*),A3DS(*),B3DF(*),B3DS(*)
      COMPLEX C3DF(*),C3DS(*)
C**************************************************************************
      IF (OCC.NE.0.0) THEN
      	PREF=PBC/100.0
        JC=NSAM/2+1
        IC=NSAM*NSAM/2
      	NSAMH=NSAM/2
      	RII=2.0*PI*RI/NSAM
      	IRREC=INT(REAL(NSAM)*RREC)
      	IF (IRREC.GT.NSAMH) IRREC=NSAMH
      	IRREC2=IRREC**2
      	TMP=(PRESA/PREF-BOFF/PBC)
        IF (TMP.GT.TLIM) THEN
      	  TMP=TLIM
C      	  PRINT *,'TLIM IN ACTION!!!!!'
      	ENDIF
      	TMP=2.0*TMP/REAL(NSAMH)**2

        CALL ROTMAT(PHI,THETA,PSI,AMAG,TM)
C
      	IF (ISIGN.LT.0) THEN
      	  DO 10 L=1,NSAM
            NL=NSAM*(L-1)
      	    DO 10 J=1,NSAM
      	      IS=J+NL
              NJ=NSAMH*((J-1)+NL)
      	      B3DS(IS)=A3DS(IS)
c      	      T3DS(IS)=S3DS(IS)
      	      DO 10 I=1,NSAM/2
      	        ID=I+NJ
      	        B3DF(ID)=A3DF(ID)
c      	        T3DF(ID)=S3DF(ID)
10	  CONTINUE
      	ENDIF
C
        DO 940 I=1,NSYM
          ISYM(I)=1
940     CONTINUE
C
900     CONTINUE
C
        DO 910 I=1,9
          DM(I)=TM(I)
910     CONTINUE
        DO 920 I=1,NSYM
          IF (ISYM(I).NE.JSYM(I)) THEN
            DO 930 J=1,ISYM(I)
              CALL MATMUL_T(DM,SYMOP(1,1,I),DM)
930         CONTINUE
          ENDIF
920     CONTINUE
C
        ST=REAL(2*JC-1)/REAL(IMP)
        DO 800 I=1,IMP
          NKSUM1(I)=0
          TTD1(I)=0.0D0
800     CONTINUE
!$OMP PARALLEL DO
c REDUCTION(+:NKSUM1,TTD1)
        DO 20 I=1,IMP
          CALL PINSERT_S(NSAM,IRAD,RII,IRREC2,SPEC,SPEQ,CTFF,
     +             CTFS,OCC,B3DF,B3DS,T3DF,T3DS,TTD1,SHX,SHY,
     +             ASUM,VSUM,PSUM,C3DF,KSUM,ISIGN,SINCLUT,IRADA,
     +             IPAD,THET,IEWALD,C3DS,NNSTAT,NKSUM1(I),JC,DM,
     +             TMP,NSAMH,IC,ST,I)
20      CONTINUE
        DO 810 I=1,IMP
          NKSUM=NKSUM+NKSUM1(I)
          TTD=TTD+TTD1(I)
810     CONTINUE
C
        I=1
990     CONTINUE
        ISYM(I)=ISYM(I)+1
        IF (ISYM(I).GT.JSYM(I)) THEN
          ISYM(I)=1
          I=I+1
          IF (I.LE.NSYM) GOTO 990
        ELSE
          I=1
          GOTO 900
        ENDIF
C
      ENDIF
C
      RETURN
      END
